app.scriptPreferences.userInteractionLevel=UserInteractionLevels.interactWithAll;

var err_msg = ""

if(app.documents.length == 0){
	err_msg = "開いているドキュメントがありません。処理を中止します";
	}
if(app.selection.length != 1){
	err_msg = "長方形をひとつ選択してください";
	}
if(err_msg == "" && app.selection[0].constructor.name != "Rectangle"){
	err_msg = "長方形でないオブジェクトが選択されています。処理を中止します";
	}
if(err_msg == "" && app.selection[0].rotationAngle != 0){
	err_msg = "回転している長方形では実行できません。処理を中止します";
	}
if(err_msg == "" && app.selection[0].shearAngle != 0){
	err_msg = "シアーのかかった長方形では実行できません。処理を中止します";
	}
if(err_msg != ""){
	alert(err_msg);
	} else {
	make_stamp();
	}

function make_stamp(){
	//使用する単位の設定
	un = ["ポイント", "インチ", "ミリメートル", "センチメートル", "歯", "アメリカ式ポイント"];
	ub = [2054188905, 2053729891, 2053991795, 2053336435, 1516790048, 1514238068];
	ux = [1, 0.0138889, 0.3528, 0.03528, 1.4112, 1.003921];
	//現在の単位を取得
	u = p_of_l(ub, app.activeDocument.viewPreferences.horizontalMeasurementUnits);
	if(u == -1) u = 2 ;//単位がリストになければミリメートルを使用
	umax = 15552 * ux[u]; //幅の最大値（とりあえずドキュメントの最大値に合わせてあります）
	
	 //選択された長方形の情報を取得
	var s = app.selection[0];
	var z = s.index; //Z-order、最前面からの順番
	
	//ここからダイアログを作成
	dlg = app.dialogs.add({name:"切手様の枠を追加"});
	dlc = dlg.dialogColumns.add();
	
	with(dlc.dialogRows.add()){ //1行目
			staticTexts.add({staticLabel:"縁取り幅："});
			var dw = realEditboxes.add({minWidth:80, minimumValue:0, maximumValue:umax, editValue:3});
			var du = dropdowns.add({minWidth:80, stringList:un, selectedIndex:u});
		}
	
	with(dlc.dialogRows.add()){ //2行目
		staticTexts.add({staticLabel:"縁取り幅に対する目打ち部分の幅："});
		var dr = percentEditboxes.add({minWidth:80, editValue:33});
		}
	
	//キャンセル時の処理
	if(!dlg.show()){
		dlg.destroy();
		return;
		}
	
	//単位の変更
	u = du.selectedIndex;
	org_h = app.activeDocument.viewPreferences.horizontalMeasurementUnits;
	org_v = app.activeDocument.viewPreferences.verticalMeasurementUnits;
	app.activeDocument.viewPreferences.horizontalMeasurementUnits = ub[u];
	app.activeDocument.viewPreferences.verticalMeasurementUnits = ub[u];
	
  	var t0 = s.visibleBounds[0] - dw.editValue; //上
 	var l0 = s.visibleBounds[1] - dw.editValue; //左
 	var b0 = s.visibleBounds[2] + dw.editValue; //下
 	var r0 = s.visibleBounds[3] + dw.editValue; //右
	var d = dr.editValue * dw.editValue / 100; //半径
	dlg.destroy(); //ダイアログの解放
	
	//ここから図形の作成
	var a = d * 0.5523; //アンカーポイントの長さ0.55228474
	t1 = t0 + d;
	l1 = l0 + d;
	b1 = b0 - d;
	r1 = r0 - d;
	t2 = t0 + a;
	l2 = l0 + a;
	b2 = b0 - a;
	r2 = r0 - a;
	//パスポイントの位置を割り出す
	var nv = Math.floor((b1 - t1)/(d * 2));
	if (nv % 2 == 0 ) nv = nv - 1;
	var dv = (b1 - t1 - d * (nv - 1))/((nv + 1) / 2);
	var nh = Math.floor((r1 - l1)/(d * 2));
	if (nh % 2 == 0 ) nh = nh - 1;
	var dh = (r1 - l1 - d * (nh  - 1))/((nh + 1) / 2);
	
	//長方形を作成
	var rec = app.activeDocument.rectangles.add({
		fillColor:"Paper", strokeColor:"None",
		geometricBounds: [t0, l0, b0, r0]
		});
	//選択された長方形の奥へ移動
	//長方形同士なのでこれだけで可能
	while (rec.index < z+1){
		rec.sendBackward();
		}
	
	//アンカーポイントの追加
	//追加した時点で長方形ではなくpolygonになる
	nv = (nv + 1) * 1.5 - 1;
	nh = (nh + 1) * 1.5 - 1;
	j = nv + nh + nv + nh - 4;
	for( i = 0 ; i < j ; i++ ) rec.paths[0].pathPoints.add();
	
	//アンカーポイントの移動
	var pp = rec.paths[0].pathPoints;
	//左辺
	j = 0;
	for ( i = 0 ; i < nv - 1 ; i++ ) {
		y = t1 + j * (d * 2 + dv);
		if (i % 3 == 0) {
			pp[i].anchor = [l0, y];
			pp[i].leftDirection = [l2, y];
		} else if (i % 3 == 1) {
			pp[i].anchor = [l0, y + dv];
			pp[i].rightDirection = [l2, y + dv];
		} else {
			pp[i].anchor = [l1, y + dv + d];
			pp[i].leftDirection = [l1,  y + dv + d - a];
			pp[i].rightDirection = [l1, y + dv + d + a];
			j += 1;
			}
		}
	pp[i].anchor = [l0, b1];
	pp[i].rightDirection = [l2, b1];
	//下辺
	j = 0;
	for ( i = nv ; i < nv + nh - 1 ; i++ ) {
		x = l1 + j * (d * 2 + dh);
		ii = i - nv;
		if (ii % 3 == 0) {
			pp[i].anchor = [x, b0];
			pp[i].leftDirection = [x, b2];
		} else if (ii % 3 == 1) {
			pp[i].anchor = [x + dh, b0];
			pp[i].rightDirection = [x + dh, b2];
		} else {
			pp[i].anchor = [x + dh + d, b1];
			pp[i].leftDirection = [x + dh + d - a, b1];
			pp[i].rightDirection = [x + dh + d + a, b1];
			j += 1;
			}
		}
	pp[i].anchor = [r1, b0];
	pp[i].rightDirection = [r1, b2];
	//右辺
	j = 0;
	for ( i = nv + nh ; i < nv + nh + nv - 1 ; i++ ) {
		y = b1 - j * (d * 2 + dv);
		ii = i - nv - nh;
		if (ii % 3 == 0) {
			pp[i].anchor = [r0, y];
			pp[i].leftDirection = [r2, y];
		} else if (ii % 3 == 1) {
			pp[i].anchor = [r0, y - dv];
			pp[i].rightDirection = [r2, y - dv];
		} else {
			pp[i].anchor = [r1, y - dv - d];
			pp[i].leftDirection = [r1,  y - dv - d + a];
			pp[i].rightDirection = [r1, y - dv - d - a];
			j += 1;
			}
		}
	pp[i].anchor = [r0, t1];
	pp[i].rightDirection = [r2, t1];
	//上辺
	j = 0;
	for ( i = nv + nh + nv ; i < nv + nh + nv + nh - 1 ; i++ ) {
		x = r1 - j * (d * 2 + dh);
		ii = i - nv - nh - nv;
		if (ii % 3 == 0) {
			pp[i].anchor = [x, t0];
			pp[i].leftDirection = [x, t2];
		} else if (ii % 3 == 1) {
			pp[i].anchor = [x - dh, t0];
			pp[i].rightDirection = [x - dh, t2];
		} else {
			pp[i].anchor = [x - dh - d, t1];
			pp[i].leftDirection = [x - dh - d + a, t1];
			pp[i].rightDirection = [x - dh - d - a, t1];
			j += 1;
			}
		}
	pp[i].anchor = [l1, t0];
	pp[i].rightDirection = [l1, t2];
	
	//単位を元に戻す
	app.activeDocument.viewPreferences.horizontalMeasurementUnits = org_h;
	app.activeDocument.viewPreferences.verticalMeasurementUnits = org_v;
 }

function p_of_l(list, valu){
	for(i = 0 ; i < list.length ; i++){
		if(list[i] == valu){
			return i;
			break;
		}
	}
}
